
/*
ShaMan Hash Shared Library (SHA1 SHA256 SHA384 SHA512 MD5 BASE64 + random string generator )
Copyright (c) 2007 James Mrad (xtremejames183@msn.com)

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "shaman.h"

/* local declaration */
static char *strCopy( char * str ) ; /* safe strcpy */

int GenRandomString(uint8_t	*output,unsigned long	len)
{
const char chars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
size_t	max = strlen(chars);
time_t	sec;
unsigned long	i = 0L;

if(output == NULL ) return -1 ;

sec = time(NULL);
/* convert to unsigned int */
srand((unsigned int)sec);

for(;i < len - 2L;++i)
		output[i] = chars[ rand() % max ] ;
output[i] = '\0' ;

return SHAMAN_OK;
}
uint8_t	*	XReadFile(FILE	*fp)
{

uint8_t *output = NULL;
  uint8_t *buffer;
  uint8_t	*oldStr;
  int bytes, byteCount = 0;
	long offs;

if(fp == NULL)	
		return NULL;
 
  

  buffer = (char *)malloc(FILE_HASH_SIZE);
	if(buffer == NULL)
		return NULL;

	 /* Ensure it is on a 64-bit boundary. */
  if ((offs = (long) buffer & 7L))
    buffer += 8 - offs;

	//bzero(buffer,FILE_HASH_SIZE);
  
  while( ( bytes = fread( buffer, sizeof( char ), FILE_HASH_SIZE - 1, fp ) ) > 0 )
  {
    if( output == NULL )
    {
      /* allocate output */
      output = ( char * )malloc( bytes + 1 );
      memcpy( output, buffer, bytes + 1 );
    }
    else
    {
      /* already exits - reallocate! */
      
      /* we cannot be sure, oldStr isn't overwritten during realloc() */
      oldStr = strCopy( output ); 
	
      output = ( char * )realloc( output, byteCount + bytes + 1 );
	  if(output == NULL )	
		  return NULL;
      strncpy( output, oldStr, byteCount );
      
      free( oldStr );
      memcpy( output + byteCount, buffer, bytes + 1 );
    }
  
    byteCount += bytes;
   bzero(buffer,FILE_HASH_SIZE);
  }
  free(buffer);
  return output;

}
static char *strCopy( char * str )
{
	char	*n;
  if( !str )
  {
    return NULL;
  }
  
 n = ( char * )malloc( strlen( str ) + 1 );
  memcpy( n, str, strlen( str ) + 1 );
  
  return n;
}
#if defined _WIN32 || !defined HAVE_BZERO

void bzero(void *buf, size_t buf_size)
{
	memset(buf, 0, buf_size);
}

#endif 

SHAMAN_CTX	*	InitShaMan(void)
{
	
	SHAMAN_CTX	*shaman_tmp ;
	pool p;

	p = pool_new();
	if( p == NULL )
			return (SHAMAN_CTX	*)NULL;
	
	shaman_tmp = (SHAMAN_CTX	*)pmalloco(p,sizeof(struct shaman_st)) ;
	/* pmalloco = malloc + bzero */
	if(shaman_tmp == NULL )
			return (SHAMAN_CTX	*)NULL;

	shaman_tmp->meth = SHA256_HASH ;

	shaman_tmp->hash = (uint8_t	*)pmalloc(p,MAX_HASH_BUFFER);
	if(shaman_tmp->hash == NULL )
	{
			pool_free(p);
			return (SHAMAN_CTX	*)NULL;
	}

	shaman_tmp->version = pstrdupx(p,SHAMAN_VER,strlen(SHAMAN_VER));
	shaman_tmp->clear = NULL;
	shaman_tmp->fp = NULL ;

	shaman_tmp->p = p ;

	return shaman_tmp;
}
int ShaManAdd(SHAMAN_CTX	*sctx,uint8_t *cstr)
{
	size_t len ;

	if(sctx == NULL || cstr == NULL || cstr[0] == '\0' )
			return -1 ;

	len = strlen(cstr);
	/* copie it to the Context */
	sctx->clear = pstrdupx(sctx->p,cstr,len);

	return SHAMAN_OK ;
}
/* Core Hash Processing */
int ShaManHashData(SHAMAN_CTX	*sctx)
{
	int i,len;
	size_t	HASHLEN;
	  union {
    SHA1Context sha1;
    SHA256Context sha256;
    SHA384Context sha384;
    SHA512Context sha512;
	md5_state_t	MD5;
  } s;
	  union{
		  uint8_t	sha1[SHA1_HASH_SIZE];
		  uint8_t	sha256[SHA256_HASH_SIZE];
		  uint8_t	sha384[SHA384_HASH_SIZE];
		  uint8_t	sha512[SHA512_HASH_SIZE];
		  uint8_t	digest[16];
	  }h;

		if(sctx == NULL ) return -1 ; /* fatal */

		if( sctx->clear == NULL || sctx->clear[0]=='\0' )
	  {
		 sctx->hash = strCopy("NULL");
		return -1;
	  }
		len = (int)strlen(sctx->clear);

		switch(sctx->meth)
			{
		case SHA1_HASH:
					SHA1Init(&s.sha1);
					SHA1Update(&s.sha1,sctx->clear,len);
					SHA1Final(&s.sha1,h.sha1);
					for(i = 0 ; i< SHA1_HASH_SIZE;)
							snprintf(sctx->hash + i *2 -2,SHA1_HASH_SIZE+7,"%02x",h.sha1[i++]);
					break;
		case SHA256_HASH:
					SHA256Init(&s.sha256);
					SHA256Update(&s.sha256,sctx->clear,len);
					SHA256Final(&s.sha256,h.sha256);
					for(i = 0 ; i< SHA256_HASH_SIZE;)
							snprintf(sctx->hash + i *2 -2,SHA256_HASH_SIZE+7,"%02x",h.sha256[i++]);
					break;
		case SHA384_HASH:
					SHA384Init(&s.sha384);
					SHA384Update(&s.sha384,sctx->clear,len);
					SHA384Final(&s.sha384,h.sha384);
					for(i = 0 ; i< SHA384_HASH_SIZE;)
							snprintf(sctx->hash + i *2 -2,SHA384_HASH_SIZE+7,"%02x",h.sha384[i++]);
					break;
		case SHA512_HASH:
					SHA512Init(&s.sha512);
					SHA512Update(&s.sha512,sctx->clear,len);
					SHA512Final(&s.sha512,h.sha512);
					for(i = 0 ; i< SHA512_HASH_SIZE;)
							snprintf(sctx->hash + i *2 -2,SHA512_HASH_SIZE+7,"%02x",h.sha512[i++]);
					break;
		case MD5_HASH:
					/**
					RFC 1321
					*/
					md5_init(&s.MD5);
					md5_append(&s.MD5,sctx->clear,len);
					md5_finish(&s.MD5,h.digest);
					for(i = 0 ; i< 16;)
							snprintf(sctx->hash + i *2-2,sizeof(h.digest),"%02x",h.digest[i++]);
					break;
				case BASE64_HASH:
					ap_base64encode(sctx->hash,sctx->clear,len);
					break;
				default:
					 sctx->hash = strCopy("NULL");
					return -1;
		}

		bzero(&h,sizeof(h));
		bzero(&s,sizeof(s));

return 0;

}
int ShaManFileCheckSum(SHAMAN_CTX	*sctx,char	*fname)
{

	uint8_t	hash[SHA512_HASH_SIZE];
	uint8_t	digest[16];
	int	hashlen,i;
	size_t len;
	off_t	offs;
	uint8_t	*buffer;
	  union {
    SHA1Context sha1;
    SHA256Context sha256;
    SHA384Context sha384;
    SHA512Context sha512;
	md5_state_t	md5;
  } s;

	  if(fname == NULL || fname[0]=='\0')
	  {
		  sctx->hash = strCopy("NULL");
		return -1;
	  }

	  /* verify File Exist
	  if(FileExists(file)== false) // this funct is part of closed-Source Project sorry ;-) 
	  {
			sctx->hash = strCopy("NULL");
			return -1;
	  }*/
#ifdef _WIN32
	  fopen_s(&sctx->fp,fname,"rb");
#else
	  sctx->fp=fopen(fname,"rb");
#endif
	  if(sctx->fp == NULL)
	  {
			/* check errno */
		  sctx->hash = strCopy("NULL");
			return -1;
	  }
	  switch(sctx->meth)
	  {
	  case SHA1_HASH:
		  SHA1Init(&s.sha1);
		  break;
	  case SHA256_HASH:
		  SHA256Init(&s.sha256);
		  break;
	  case SHA384_HASH:
		  SHA384Init(&s.sha384);
		  break;
	  case SHA512_HASH:
		  SHA512Init(&s.sha512);
		  break;
	  case MD5_HASH:
		 md5_init(&s.md5);
		  break;
	  default:
		  sctx->hash = strCopy("NULL");
		return -1;
	  }

	  buffer = (char *)pmalloco(sctx->p,(FILE_HASH_SIZE + 7));
	if(buffer == NULL)
	{
		sctx->hash = strCopy("NULL");
		return -1;
	}
	 /* Ensure it is on a 64-bit boundary. */
  if ((offs = (long) buffer & 7L))
    buffer += 8 - offs;
	 
	  
	  while ((len = fread (buffer, 1, FILE_HASH_SIZE, sctx->fp)) > 0) {
		  switch(sctx->meth)
		{
		case 0:
			SHA1Update(&s.sha1,buffer,len);
			SHA1Final(&s.sha1,hash);
			hashlen = SHA1_HASH_SIZE;
			break;
		case 1:
			SHA256Update(&s.sha256,buffer,len);
			SHA256Final(&s.sha256,hash);
			hashlen = SHA256_HASH_SIZE;
			break;
		case 2:
			SHA384Update(&s.sha384,buffer,len);
			SHA384Final(&s.sha384,hash);
			hashlen = SHA384_HASH_SIZE;
			break;
		case 3:
			SHA512Update(&s.sha512,buffer,len);
			SHA512Final(&s.sha512,hash);
			hashlen = SHA512_HASH_SIZE ;
			break;
		case 4:
			/**
					TODO: NOT STABLE MUST VERIFY
					*/
			md5_append(&s.md5,buffer,len);
			md5_finish(&s.md5,digest);
			hashlen = 16 ;
			break;
		default:
			sctx->hash = strCopy("NULL");
			return -1;
		}

	  }

	//bzero(&hex,hashlen+7);

	/* print our code */
	  switch(sctx->meth)
	 {
	 case 0:
	 case 1:
	 case 2:
	 case 3:
		for(i = 0; i < hashlen ;)
			snprintf(sctx->hash + i*2-2,hashlen,"%02x",hash[i++]);
	break;
	 case 4:
		 for(i = 0; i < hashlen ;)
			 snprintf(sctx->hash + i*2-2,hashlen,"%02x",digest[i++]);
	break;

	 }
	fclose(sctx->fp);
	bzero(&s,sizeof(s));
	free(buffer);

	return 0;
}


int DestroyShaMan(SHAMAN_CTX	*sctx)
{
	if(sctx == NULL ) 
		return SHAMAN_OK ; /* already destroyed */ 

	if(sctx->fp != NULL ) 
			fclose(sctx->fp);
/*
	if(sctx->clear != NULL ) // safe crash
		free(sctx->clear);
	if(sctx->hash != NULL )
		free(sctx->hash);

		pool_free(sctx->p);

-*/
	return SHAMAN_OK;
}


/* ShaCryptString : Cette Fonction crypte une string selon un des algorithmes choisis et retourne 0 en cas de russite, -1 en cas d'erreur.
arg0 : Buffer qui contiendra le rsultat du cryptage
arg1: Chane  Crypter
arg2: Type de HASH (SHA1, SHA256, MD5...)*/
int ShaCryptString(char* buf, uint8_t* cstr, double hash)
{
	SHAMAN_CTX	*SCT; /* Context ShaMan */
	int res,i ;
	uint8_t	test[512];
	char HAMETH[6][10] = { "SHA1","SHA256","SHA384","SHA512","MD5","BASE64" } ;

	/* Init Shaman Library*/
	SCT = InitShaMan() ;
	
	if( SCT == NULL)
	{
		return EXIT_FAILURE;
	}
	

	/* Prparation au Hashage */
	ShaManAdd(SCT, cstr);

	/* Hashage */
	SCT->meth = hash ;
	res = ShaManHashData(SCT);

		if(res != SHAMAN_OK)
		{
		 return -1; //Retourner -1 si erreur de hashage.
		}

	//Copier le rsultat du hashage  l'emplacement mmoire du buffer.
	strcpy(buf,SCT->hash);
	DestroyShaMan(SCT);
	return 0;
}